@@ -43,18 +43,38 @@ class CornFlow(object):
 
         return wrapper
 
+    def prepare_encoding(func):
+        @wraps(func)
+        def wrapper(*args, **kwargs):
+            encoding = kwargs.get("encoding", "br")
+            if encoding not in [
+                "gzip",
+                "compress",
+                "deflate",
+                "br",
+                "identity",
+            ]:
+                encoding = "br"
+            kwargs["encoding"] = encoding
+            result = func(*args, **kwargs)
+            return result
+
+        return wrapper
+
     # def expect_201(func):
     #     return partial(expect_status, status=201)
     #
     # def expect_200(func):
     #     return partial(expect_status, status=200)
 
-    def api_for_id(self, api, id, method, post_url="", **kwargs):
+    def api_for_id(self, api, id, method, post_url="", encoding=None, **kwargs):
         """
         :param api: the resource in the server
         :param id: the id of the particular object
         :param method: HTTP method to apply
         :param post_url: optional action to apply
+        :param encoding: optional string with the type of encoding, if it is not specified it uses br encoding,
+        options are: gzip, compress, deflate, br or identity
         :param kwargs: other arguments to requests.request
 
         :return: requests.request
@@ -65,77 +85,104 @@ class CornFlow(object):
         return requests.request(
             method=method,
             url=url,
-            headers={"Authorization": "access_token " + self.token},
+            headers={
+                "Authorization": "access_token " + self.token,
+                "Content-Encoding": encoding,
+            },
             **kwargs
         )
 
-    def get_api(self, api, method="GET", **kwargs):
+    def get_api(self, api, method="GET", encoding=None, **kwargs):
         return requests.request(
             method=method,
             url=urljoin(self.url, api) + "/",
-            headers={"Authorization": "access_token " + self.token},
+            headers={
+                "Authorization": "access_token " + self.token,
+                "Content-Encoding": encoding,
+            },
             **kwargs
         )
 
     @ask_token
-    def get_api_for_id(self, api, id, post_url="", **kwargs):
+    @prepare_encoding
+    def get_api_for_id(self, api, id, post_url="", encoding=None, **kwargs):
         """
         api_for_id with a GET request
         """
+
         return self.api_for_id(
-            api=api, id=id, post_url=post_url, **kwargs, method="get"
+            api=api, id=id, post_url=post_url, encoding=encoding, **kwargs, method="get"
         )
 
     @ask_token
-    def delete_api_for_id(self, api, id, **kwargs):
+    @prepare_encoding
+    def delete_api_for_id(self, api, id, encoding=None, **kwargs):
         """
         api_for_id with a DELETE request
         """
-        return self.api_for_id(api=api, id=id, **kwargs, method="delete")
+        return self.api_for_id(
+            api=api, id=id, encoding=encoding, **kwargs, method="delete"
+        )
 
     @ask_token
-    def put_api_for_id(self, api, id, payload, **kwargs):
+    @prepare_encoding
+    def put_api_for_id(self, api, id, payload, encoding=None, **kwargs):
         """
         api_for_id with a PUT request
         """
-        return self.api_for_id(api=api, id=id, json=payload, method="put", **kwargs)
+        return self.api_for_id(
+            api=api, id=id, json=payload, method="put", encoding=encoding, **kwargs
+        )
 
     @ask_token
-    def patch_api_for_id(self, api, id, payload, **kwargs):
+    @prepare_encoding
+    def patch_api_for_id(self, api, id, payload, encoding=None, **kwargs):
         """
         api_for_id with a PATCH request
         """
-        return self.api_for_id(api=api, id=id, json=payload, method="patch", **kwargs)
+        return self.api_for_id(
+            api=api, id=id, json=payload, method="patch", encoding=encoding ** kwargs
+        )
 
     @ask_token
-    def post_api_for_id(self, api, id, **kwargs):
+    @prepare_encoding
+    def post_api_for_id(self, api, id, encoding=None, **kwargs):
         """
         api_for_id with a POST request
         """
-        return self.api_for_id(api=api, id=id, method="post", **kwargs)
+        return self.api_for_id(
+            api=api, id=id, method="post", encoding=encoding, **kwargs
+        )
 
     @ask_token
-    def create_api(self, api, **kwargs):
+    @prepare_encoding
+    def create_api(self, api, encoding=None, **kwargs):
         return requests.post(
             urljoin(self.url, api),
-            headers={"Authorization": "access_token " + self.token},
+            headers={
+                "Authorization": "access_token " + self.token,
+                "Content-Encoding": encoding,
+            },
             **kwargs
         )
 
     @log_call
-    def sign_up(self, username, email, pwd):
+    @prepare_encoding
+    def sign_up(self, username, email, pwd, encoding=None):
         """
         Sign-up to the server. Creates a new user or returns error.
 
         :param str username: username
         :param str email: email
         :param str pwd: password
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
 
         :return: a dictionary with a token inside
         """
         return requests.post(
             urljoin(self.url, "signup/"),
             json={"username": username, "password": pwd, "email": email},
+            headers={"Content-Encoding": encoding},
         )
 
     @log_call
@@ -152,17 +199,21 @@ class CornFlow(object):
             )
         )
 
-    def login(self, username, pwd):
+    @prepare_encoding
+    def login(self, username, pwd, encoding=None):
         """
         Log-in to the server.
 
         :param str username: username
         :param str pwd: password
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
 
         :return: a dictionary with a token inside
         """
         response = requests.post(
-            urljoin(self.url, "login/"), json={"username": username, "password": pwd}
+            urljoin(self.url, "login/"),
+            json={"username": username, "password": pwd},
+            headers={"Content-Encoding": encoding},
         )
         if response.status_code == 200:
             result = response.json()
@@ -178,8 +229,9 @@ class CornFlow(object):
     # TODO: those status_code checks should be done via a decorator. But I do not know how.
     @ask_token
     @log_call
+    @prepare_encoding
     def create_instance(
-        self, data, name=None, description="", schema="solve_model_dag"
+        self, data, name=None, description="", schema="solve_model_dag", encoding=None
     ):
         """
         Uploads an instance to the server
@@ -188,6 +240,7 @@ class CornFlow(object):
         :param str name: name for instance
         :param str description: description of the instance
         :param str schema: name of problem to solve
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
         if name is None:
             try:
@@ -195,7 +248,7 @@ class CornFlow(object):
             except IndexError:
                 raise CornFlowApiError("The `name` argument needs to be filled")
         payload = dict(data=data, name=name, description=description, schema=schema)
-        response = self.create_api("instance/", json=payload)
+        response = self.create_api("instance/", json=payload, encoding=encoding)
         if response.status_code != 201:
             raise CornFlowApiError(
                 "Expected a code 201, got a {} error instead: {}".format(
@@ -204,8 +257,18 @@ class CornFlow(object):
             )
         return response.json()
 
+    @ask_token
+    @log_call
+    @prepare_encoding
     def create_case(
-        self, name, schema, data=None, parent_id=None, description="", solution=None
+        self,
+        name,
+        schema,
+        data=None,
+        parent_id=None,
+        description="",
+        solution=None,
+        encoding=None,
     ):
         """
         Uploads a case to the server
@@ -216,6 +279,7 @@ class CornFlow(object):
         :param str schema: name of problem of case
         :param str parent_id: id of the parent directory in the tree structure
         :param dict solution: optional solution data to store inside case
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
         payload = dict(
             name=name,
@@ -230,7 +294,7 @@ class CornFlow(object):
             payload["data"] = data
         if solution is not None:
             payload["solution"] = solution
-        response = self.create_api("case/", json=payload)
+        response = self.create_api("case/", json=payload, encoding=encoding)
         if response.status_code != 201:
             raise CornFlowApiError(
                 "Expected a code 201, got a {} error instead: {}".format(
@@ -241,19 +305,24 @@ class CornFlow(object):
 
     @ask_token
     @log_call
-    def create_instance_file(self, filename, name, description="", minimize=True):
+    @prepare_encoding
+    def create_instance_file(
+        self, filename, name, description="", minimize=True, encoding=None
+    ):
         """
         Uploads a file to the server to be parsed into an instance
 
         :param str filename: path to filename to upload
         :param str name: name for instance
         :param str description: description of the instance
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
         with open(filename, "rb") as file:
             response = self.create_api(
                 "instancefile/",
                 data=dict(name=name, description=description, minimize=minimize),
                 files=dict(file=file),
+                encoding=encoding,
             )
 
         if response.status_code != 201:
@@ -266,6 +335,7 @@ class CornFlow(object):
 
     @log_call
     @ask_token
+    @prepare_encoding
     def create_execution(
         self,
         instance_id,
@@ -273,6 +343,7 @@ class CornFlow(object):
         name="test1",
         description="",
         schema="solve_model_dag",
+        encoding=None,
     ):
         """
         Creates an execution from a (previously) uploaded instance
@@ -282,6 +353,7 @@ class CornFlow(object):
         :param str description: description of the execution
         :param dict config: execution configuration
         :param str schema: name of the problem to solve
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
         payload = dict(
             config=config,
@@ -290,7 +362,7 @@ class CornFlow(object):
             description=description,
             schema=schema,
         )
-        response = self.create_api("execution/", json=payload)
+        response = self.create_api("execution/", json=payload, encoding=encoding)
         if response.status_code != 201:
             raise CornFlowApiError(
                 "Expected a code 201, got a {} error instead: {}".format(
@@ -300,24 +372,30 @@ class CornFlow(object):
         return response.json()
 
     @ask_token
-    def get_data(self, execution_id):
+    @prepare_encoding
+    def get_data(self, execution_id, encoding=None):
         """
         Downloads the data from an execution
 
         :param str execution_id: id for the execution
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.get_api_for_id(api="dag/", id=execution_id)
+        response = self.get_api_for_id(api="dag/", id=execution_id, encoding=encoding)
         return response.json()
 
     @ask_token
-    def write_solution(self, execution_id, **kwargs):
+    @prepare_encoding
+    def write_solution(self, execution_id, encoding=None, **kwargs):
         """
         Edits an execution
 
         :param str execution_id: id for the execution
         :param kwargs: optional data to edit
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.put_api_for_id("dag/", id=execution_id, payload=kwargs)
+        response = self.put_api_for_id(
+            "dag/", id=execution_id, encoding=encoding, payload=kwargs
+        )
         if response.status_code != 200:
             raise CornFlowApiError(
                 "Expected a code 200, got a {} error instead: {}".format(
@@ -328,13 +406,17 @@ class CornFlow(object):
 
     @log_call
     @ask_token
-    def stop_execution(self, execution_id):
+    @prepare_encoding
+    def stop_execution(self, execution_id, encoding=None):
         """
         Interrupts an ongoing execution
 
         :param str execution_id: id for the execution
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.post_api_for_id(api="execution/", id=execution_id)
+        response = self.post_api_for_id(
+            api="execution/", id=execution_id, encoding=encoding
+        )
         if response.status_code != 200:
             raise CornFlowApiError(
                 "Expected a code 200, got a {} error instead: {}".format(
@@ -344,7 +426,8 @@ class CornFlow(object):
         return response.json()
 
     @ask_token
-    def manual_execution(self, instance_id, config, name, **kwargs):
+    @prepare_encoding
+    def manual_execution(self, instance_id, config, name, encoding=None, **kwargs):
         """
         Uploads an execution from solution data
 
@@ -353,10 +436,11 @@ class CornFlow(object):
         :param str description: description of the execution
         :param dict config: execution configuration
         :param str schema: name of the problem to solve
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         :param kwargs: contents of the solution (inside data)
         """
         payload = dict(config=config, instance_id=instance_id, name=name, **kwargs)
-        response = self.create_api("dag/", json=payload)
+        response = self.create_api("dag/", json=payload, encoding=encoding)
         if response.status_code != 201:
             raise CornFlowApiError(
                 "Expected a code 201, got a {} error instead: {}".format(
@@ -367,186 +451,263 @@ class CornFlow(object):
 
     @log_call
     @ask_token
-    def get_results(self, execution_id):
+    @prepare_encoding
+    def get_results(self, execution_id, encoding=None):
         """
         Downloads results from an execution
 
         :param str execution_id: id for the execution
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.get_api_for_id(api="execution/", id=execution_id)
+        response = self.get_api_for_id(
+            api="execution/", id=execution_id, encoding=encoding
+        )
         return response.json()
 
     @log_call
     @ask_token
-    def get_status(self, execution_id):
+    @prepare_encoding
+    def get_status(self, execution_id, encoding=None):
         """
         Downloads the current status of an execution
 
         :param str execution_id: id for the execution
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
         response = self.get_api_for_id(
-            api="execution/", id=execution_id, post_url="status"
+            api="execution/", id=execution_id, post_url="status", encoding=encoding
         )
         return response.json()
 
     @log_call
     @ask_token
-    def get_log(self, execution_id):
+    @prepare_encoding
+    def get_log(self, execution_id, encoding=None):
         """
         Downloads the log for an execution
 
         :param str execution_id: id for the execution
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
         response = self.get_api_for_id(
-            api="execution/", id=execution_id, post_url="log"
+            api="execution/", id=execution_id, post_url="log", encoding=encoding
         )
         return response.json()
 
     @log_call
     @ask_token
-    def get_solution(self, execution_id):
+    @prepare_encoding
+    def get_solution(self, execution_id, encoding=None):
         """
         Downloads the solution data for an execution
 
         :param str execution_id: id for the execution
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
         response = self.get_api_for_id(
-            api="execution/", id=execution_id, post_url="data"
+            api="execution/", id=execution_id, post_url="data", encoding=encoding
         )
         return response.json()
 
     @log_call
     @ask_token
-    def get_all_instances(self, params=None):
+    @prepare_encoding
+    def get_all_instances(self, params=None, encoding=None):
         """
         Downloads all the user's instances
 
         :param dict params: optional filters
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        return self.get_api("instance", params=params).json()
+        return self.get_api("instance", params=params, encoding=encoding).json()
 
     @log_call
     @ask_token
-    def get_all_cases(self, params=None):
+    @prepare_encoding
+    def get_all_cases(self, params=None, encoding=None):
         """
         Downloads all the user's cases
 
         :param dict params: optional filters
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        return self.get_api("case", params=params).json()
+        return self.get_api("case", params=params, encoding=encoding).json()
 
     @log_call
     @ask_token
-    def get_all_executions(self, params=None):
+    @prepare_encoding
+    def get_all_executions(self, params=None, encoding=None):
         """
         Downloads all the user's executions
 
         :param dict params: optional filters
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        return self.get_api("execution", params=params).json()
+        return self.get_api("execution", params=params, encoding=encoding).json()
 
     @log_call
     @ask_token
-    def get_all_users(self):
+    @prepare_encoding
+    def get_all_users(self, encoding=None):
         """
         Downloads all the users in the server
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        return self.get_api("user").json()
+        return self.get_api("user", encoding=encoding).json()
 
     @log_call
     @ask_token
-    def get_one_user(self, user_id):
+    @prepare_encoding
+    def get_one_user(self, user_id, encoding=None):
         """
         Downloads all information on a user
 
         :param str user_id: user id
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        return self.get_api_for_id(api="user", id=user_id)
+        return self.get_api_for_id(api="user", id=user_id, encoding=encoding)
 
     @log_call
     @ask_token
-    def get_one_instance(self, reference_id):
+    @prepare_encoding
+    def get_one_instance(self, reference_id, encoding=None):
         """
         Downloads header of an instance
 
         :param str reference_id: id for the instance
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
+
         """
-        response = self.get_api_for_id(api="instance", id=reference_id)
+        response = self.get_api_for_id(
+            api="instance", id=reference_id, encoding=encoding
+        )
         return response.json()
 
     @log_call
     @ask_token
-    def get_one_case(self, reference_id):
+    @prepare_encoding
+    def get_one_case(self, reference_id, encoding=None):
         """
         Downloads header of a case
 
         :param str reference_id: id for the case
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.get_api_for_id(api="case", id=reference_id)
+        response = self.get_api_for_id(api="case", id=reference_id, encoding=encoding)
         return response.json()
 
     @log_call
     @ask_token
-    def delete_one_case(self, reference_id):
+    @prepare_encoding
+    def delete_one_case(self, reference_id, encoding=None):
         """
         Deletes a case
 
         :param str reference_id: id for the case
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.delete_api_for_id(api="case", id=reference_id)
+        response = self.delete_api_for_id(
+            api="case", id=reference_id, encoding=encoding
+        )
         return response.json()
 
     @log_call
     @ask_token
-    def put_one_case(self, reference_id, payload):
+    @prepare_encoding
+    def put_one_case(self, reference_id, payload, encoding=None):
         """
         Edits a case
 
         :param str reference_id: id for the case
         :param dict payload: data to edit
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.put_api_for_id(api="case", id=reference_id, payload=payload)
+        response = self.put_api_for_id(
+            api="case", id=reference_id, payload=payload, encoding=encoding
+        )
         return response.json()
 
     @log_call
     @ask_token
-    def patch_one_case(self, reference_id, payload):
+    @prepare_encoding
+    def patch_one_case(self, reference_id, payload, encoding=None):
         """
         Patch a case
 
         :param str reference_id: id for the case
         :param dict payload: data to edit
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.patch_api_for_id(api="case", id=reference_id, payload=payload)
+        response = self.patch_api_for_id(
+            api="case", id=reference_id, payload=payload, encoding=encoding
+        )
         return response.json()
 
     @log_call
     @ask_token
-    def delete_one_instance(self, reference_id):
+    @prepare_encoding
+    def delete_one_instance(self, reference_id, encoding=None):
         """
         Delete an instance
 
         :param str reference_id: id for the instance
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.delete_api_for_id(api="instance", id=reference_id)
+        response = self.delete_api_for_id(
+            api="instance", id=reference_id, encoding=encoding
+        )
         return response
 
     @ask_token
-    def get_schema(self, dag_name):
+    @prepare_encoding
+    def get_schema(self, dag_name, encoding=None):
         """
         Downloads schemas for a problem
 
         :param str dag_name: id for the problem
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        response = self.get_api_for_id(api="schema", id=dag_name)
+        response = self.get_api_for_id(api="schema", id=dag_name, encoding=encoding)
         return response.json()
 
     @ask_token
-    def get_all_schemas(self):
+    @prepare_encoding
+    def get_all_schemas(self, encoding=None):
         """
         Downloads all problems' (aka as app's) names
+
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
         """
-        return self.get_api("schema").json()
+        return self.get_api("schema", encoding=encoding).json()
+
+    @log_call
+    @ask_token
+    @prepare_encoding
+    def get_deployed_dags(self, encoding=None):
+        """
+        Downloads the deployed dags in cornflow in order to update them
+
+        :param str encoding: the type of encoding used in the call. Defaults to 'br'
+        """
+        return self.get_api("dag/deployed/", encoding=encoding).json()
+
+    @log_call
+    @ask_token
+    @prepare_encoding
+    def create_deployed_dag(
+        self, name: str = None, description: str = None, encoding=None
+    ):
+        if name is None:
+            return {"error": "No dag name was given"}
+        payload = dict(id=name, description=description)
+        response = self.create_api("dag/deployed/", json=payload, encoding=encoding)
+        if response.status_code != 201:
+            raise CornFlowApiError(
+                "Expected a code 201, got a {} error instead: {}".format(
+                    response.status_code, response.text
+                )
+            )
+        return response.json()
 
 
 class CornFlowApiError(Exception):
